@@ -14,7 +14,7 @@ import os.path
 import time
 import re as regex
 import datetime as pyDT
-import codecs
+import io
 import thread
 import threading
 import logging
@@ -48,6 +48,7 @@ from Gnumed.business import gmXdtMappings
 from Gnumed.business import gmProviderInbox
 from Gnumed.business import gmExportArea
 from Gnumed.business import gmBilling
+from Gnumed.business import gmAutoHints
 from Gnumed.business.gmDocuments import cDocumentFolder
 from Gnumed.business.gmChartPulling import tui_chart_puller
 
@@ -83,7 +84,7 @@ def external_id_exists(pk_issuer, value):
 	return rows[0][0]
 
 #============================================================
-def person_exists(lastnames, dob, firstnames=None):
+def person_exists(lastnames, dob, firstnames=None, active_only=True):
 	args = {
 		'last': lastnames,
 		'dob': dob
@@ -97,7 +98,10 @@ def person_exists(lastnames, dob, firstnames=None):
 			#where_parts.append(u"position(%(first)s in firstnames) = 1")
 			where_parts.append(u"firstnames ~* %(first)s")
 			args['first'] = u'\\m' + firstnames
-	cmd = u"""SELECT COUNT(1) FROM dem.v_basic_person WHERE %s""" % u' AND '.join(where_parts)
+	if active_only:
+		cmd = u"""SELECT COUNT(1) FROM dem.v_active_persons WHERE %s""" % u' AND '.join(where_parts)
+	else:
+		cmd = u"""SELECT COUNT(1) FROM dem.v_all_persons WHERE %s""" % u' AND '.join(where_parts)
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
 	return rows[0][0]
 
@@ -203,9 +207,10 @@ class cDTO_person(object):
 			where_snippets.append(u'lower(gender) = lower(%(sex)s)')
 			args['sex'] = self.gender
 
+		# FIXME: allow disabled persons ?
 		cmd = u"""
 			SELECT *, '%s' AS match_type
-			FROM dem.v_basic_person
+			FROM dem.v_active_persons
 			WHERE
 				pk_identity IN (
 					SELECT pk_identity FROM dem.v_person_names WHERE %s
@@ -231,7 +236,7 @@ class cDTO_person(object):
 				return None
 			identities = [ident]
 		else:
-			identities = [ cIdentity(row = {'pk_field': 'pk_identity', 'data': row, 'idx': idx}) for row in rows ]
+			identities = [ cPerson(row = {'pk_field': 'pk_identity', 'data': row, 'idx': idx}) for row in rows ]
 
 		return identities
 	#--------------------------------------------------------
@@ -262,7 +267,7 @@ class cDTO_person(object):
 					issuer = ext_id['issuer'],
 					comment = ext_id['comment']
 				)
-			except StandardError:
+			except Exception:
 				_log.exception('cannot import <external ID> from external data source')
 				gmLog2.log_stack_trace()
 
@@ -272,7 +277,7 @@ class cDTO_person(object):
 					comm_medium = comm['channel'],
 					url = comm['url']
 				)
-			except StandardError:
+			except Exception:
 				_log.exception('cannot import <comm channel> from external data source')
 				gmLog2.log_stack_trace()
 
@@ -288,7 +293,7 @@ class cDTO_person(object):
 					region_code = adr['region_code'],
 					country_code = adr['country_code']
 				)
-			except StandardError:
+			except Exception:
 				_log.exception('cannot import <address> from external data source')
 				gmLog2.log_stack_trace()
 
@@ -440,18 +445,19 @@ class cPersonName(gmBusinessDBObject.cBusinessDBObject):
 	description = property(_get_description, lambda x:x)
 
 #============================================================
-class cIdentity(gmBusinessDBObject.cBusinessDBObject):
-	_cmd_fetch_payload = u"SELECT * FROM dem.v_basic_person WHERE pk_identity = %s"
+_SQL_get_active_person = u"SELECT * FROM dem.v_active_persons WHERE pk_identity = %s"
+_SQL_get_any_person = u"SELECT * FROM dem.v_all_persons WHERE pk_identity = %s"
+
+class cPerson(gmBusinessDBObject.cBusinessDBObject):
+	_cmd_fetch_payload = _SQL_get_any_person
 	_cmds_store_payload = [
 		u"""UPDATE dem.identity SET
 				gender = %(gender)s,
 				dob = %(dob)s,
 				dob_is_estimated = %(dob_is_estimated)s,
 				tob = %(tob)s,
-				cob = gm.nullify_empty_string(%(cob)s),
 				title = gm.nullify_empty_string(%(title)s),
 				fk_marital_status = %(pk_marital_status)s,
-				pupic = gm.nullify_empty_string(%(pupic)s),
 				deceased = %(deceased)s,
 				emergency_contact = gm.nullify_empty_string(%(emergency_contact)s),
 				fk_emergency_contact = %(pk_emergency_contact)s,
@@ -467,10 +473,8 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		"title",
 		"dob",
 		"tob",
-		"cob",
 		"gender",
 		"pk_marital_status",
-		"pupic",
 		'deceased',
 		'emergency_contact',
 		'pk_emergency_contact',
@@ -485,6 +489,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		raise AttributeError('setting ID of identity is not allowed')
 
 	ID = property(_get_ID, _set_ID)
+
 	#--------------------------------------------------------
 	def __setitem__(self, attribute, value):
 
@@ -513,9 +518,11 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 						return
 
 		gmBusinessDBObject.cBusinessDBObject.__setitem__(self, attribute, value)
+
 	#--------------------------------------------------------
 	def cleanup(self):
 		pass
+
 	#--------------------------------------------------------
 	def _get_is_patient(self):
 		return identity_is_patient(self._payload[self._idx['pk_identity']])
@@ -526,11 +533,13 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		return False
 
 	is_patient = property(_get_is_patient, _set_is_patient)
+
 	#--------------------------------------------------------
 	def _get_as_patient(self):
 		return cPatient(self._payload[self._idx['pk_identity']])
 
 	as_patient = property(_get_as_patient, lambda x:x)
+
 	#--------------------------------------------------------
 	def _get_staff_id(self):
 		cmd = u"SELECT pk FROM dem.staff WHERE fk_identity = %(pk)s"
@@ -541,6 +550,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		return rows[0][0]
 
 	staff_id = property(_get_staff_id, lambda x:x)
+
 	#--------------------------------------------------------
 	# identity API
 	#--------------------------------------------------------
@@ -592,17 +602,26 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		names = [ cPersonName(row = {'idx': idx, 'data': r, 'pk_field': 'pk_name'}) for r in rows ]
 		return names
 	#--------------------------------------------------------
-	def get_description_gender(self):
-		return _(u'%(last)s,%(title)s %(first)s%(nick)s (%(sex)s)') % {
+	def get_description_gender(self, with_nickname=True):
+		if with_nickname:
+			template = _(u'%(last)s,%(title)s %(first)s%(nick)s (%(sex)s)')
+		else:
+			template = _(u'%(last)s,%(title)s %(first)s (%(sex)s)')
+		return template % {
 			'last': self._payload[self._idx['lastnames']],
 			'title': gmTools.coalesce(self._payload[self._idx['title']], u'', u' %s'),
 			'first': self._payload[self._idx['firstnames']],
 			'nick': gmTools.coalesce(self._payload[self._idx['preferred']], u'', u" '%s'"),
 			'sex': self.gender_symbol
 		}
+
 	#--------------------------------------------------------
-	def get_description(self):
-		return _(u'%(last)s,%(title)s %(first)s%(nick)s') % {
+	def get_description(self, with_nickname=True):
+		if with_nickname:
+			template = _(u'%(last)s,%(title)s %(first)s%(nick)s')
+		else:
+			template = _(u'%(last)s,%(title)s %(first)s')
+		return template % {
 			'last': self._payload[self._idx['lastnames']],
 			'title': gmTools.coalesce(self._payload[self._idx['title']], u'', u' %s'),
 			'first': self._payload[self._idx['firstnames']],
@@ -643,7 +662,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 			return True
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': u"SELECT dem.set_nickname(%s, %s)", 'args': [self.ID, nickname]}])
 		# setting nickname doesn't change dem.identity, so other fields
-		# of dem.v_basic_person do not get changed as a consequence of
+		# of dem.v_active_persons do not get changed as a consequence of
 		# setting the nickname, hence locally setting nickname matches
 		# in-database reality
 		self._payload[self._idx['preferred']] = nickname
@@ -657,10 +676,10 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		else:
 			order_by = u'ORDER BY %s' % order_by
 
-		cmd = gmDemographicRecord._SQL_get_identity_tags % (u'pk_identity = %%(pat)s %s' % order_by)
+		cmd = gmDemographicRecord._SQL_get_person_tags % (u'pk_identity = %%(pat)s %s' % order_by)
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {u'pat': self.ID}}], get_col_idx = True)
 
-		return [ gmDemographicRecord.cIdentityTag(row = {'data': r, 'idx': idx, 'pk_field': 'pk_identity_tag'}) for r in rows ]
+		return [ gmDemographicRecord.cPersonTag(row = {'data': r, 'idx': idx, 'pk_field': 'pk_identity_tag'}) for r in rows ]
 
 	tags = property(get_tags, lambda x:x)
 
@@ -675,7 +694,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		cmd = u"SELECT pk FROM dem.identity_tag WHERE fk_tag = %(tag)s AND fk_identity = %(identity)s"
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
 		if len(rows) > 0:
-			return gmDemographicRecord.cIdentityTag(aPK_obj = rows[0]['pk'])
+			return gmDemographicRecord.cPersonTag(aPK_obj = rows[0]['pk'])
 
 		# no, add
 		cmd = u"""
@@ -689,7 +708,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 			RETURNING pk
 		"""
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True, get_col_idx = False)
-		return gmDemographicRecord.cIdentityTag(aPK_obj = rows[0]['pk'])
+		return gmDemographicRecord.cPersonTag(aPK_obj = rows[0]['pk'])
 
 	#--------------------------------------------------------
 	def remove_tag(self, tag):
@@ -701,7 +720,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 	#
 	# since external IDs are not treated as first class
 	# citizens (classes in their own right, that is), we
-	# handle them *entirely* within cIdentity, also they
+	# handle them *entirely* within cPerson, also they
 	# only make sense with one single person (like names)
 	# and are not reused (like addresses), so they are
 	# truly added/deleted, not just linked/unlinked
@@ -711,7 +730,6 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 
 		creates ID type if necessary
 		"""
-
 		# check for existing ID
 		if pk_type is not None:
 			cmd = u"""
@@ -799,6 +817,7 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 		"""
 		args = {'pk': pk_id, 'value': value, 'type': type, 'issuer': issuer, 'comment': comment}
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+
 	#--------------------------------------------------------
 	def get_external_ids(self, id_type=None, issuer=None):
 		where_parts = ['pk_identity = %(pat)s']
@@ -822,11 +841,71 @@ class cIdentity(gmBusinessDBObject.cBusinessDBObject):
 	#--------------------------------------------------------
 	def delete_external_id(self, pk_ext_id=None):
 		cmd = u"""
-delete from dem.lnk_identity2ext_id
-where id_identity = %(pat)s and id = %(pk)s"""
+			DELETE FROM dem.lnk_identity2ext_id
+			WHERE id_identity = %(pat)s AND id = %(pk)s"""
 		args = {'pat': self.ID, 'pk': pk_ext_id}
 		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 
+	#--------------------------------------------------------
+	def suggest_external_id(self, target=None, encoding=None):
+		name = self.active_name
+		last = u' '.join(p for p in name['lastnames'].split(u"-"))
+		last = u' '.join(p for p in last.split(u"."))
+		last = u' '.join(p for p in last.split(u"'"))
+		last = u''.join(gmTools.capitalize(text = p, mode = gmTools.CAPS_FIRST_ONLY) for p in last.split(u' '))
+		first = u' '.join(p for p in name['firstnames'].split(u"-"))
+		first = u' '.join(p for p in first.split(u"."))
+		first = u' '.join(p for p in first.split(u"'"))
+		first = u''.join(gmTools.capitalize(text = p, mode = gmTools.CAPS_FIRST_ONLY) for p in first.split(u' '))
+		suggestion = u'GMd-%s%s%s%s%s' % (
+			gmTools.coalesce(target, u'', u'%s-'),
+			last,
+			first,
+			self.get_formatted_dob(format = '-%Y%m%d', none_string = u''),
+			gmTools.coalesce(self['gender'], u'', u'-%s')
+		)
+		try:
+			import unidecode
+			return unidecode.unidecode(suggestion)
+		except ImportError:
+			_log.debug('cannot transliterate external ID suggestion, <unidecode> module not installed')
+		if encoding is None:
+			return suggestion
+		return suggestion.encode(encoding)
+
+	external_id_suggestion = property(suggest_external_id, lambda x:x)
+
+	#--------------------------------------------------------
+	def suggest_external_ids(self, target=None, encoding=None):
+		names2use = [self.active_name]
+		names2use.extend(self.get_names(active_only = False, exclude_active = True))
+		target = gmTools.coalesce(target, u'', u'%s-')
+		dob = self.get_formatted_dob(format = '-%Y%m%d', none_string = u'')
+		gender = gmTools.coalesce(self['gender'], u'', u'-%s')
+		suggestions = []
+		for name in names2use:
+			last = u' '.join(p for p in name['lastnames'].split(u"-"))
+			last = u' '.join(p for p in last.split(u"."))
+			last = u' '.join(p for p in last.split(u"'"))
+			last = u''.join(gmTools.capitalize(text = p, mode = gmTools.CAPS_FIRST_ONLY) for p in last.split(u' '))
+			first = u' '.join(p for p in name['firstnames'].split(u"-"))
+			first = u' '.join(p for p in first.split(u"."))
+			first = u' '.join(p for p in first.split(u"'"))
+			first = u''.join(gmTools.capitalize(text = p, mode = gmTools.CAPS_FIRST_ONLY) for p in first.split(u' '))
+			suggestion = u'GMd-%s%s%s%s%s' % (target, last, first, dob, gender)
+			try:
+				import unidecode
+				suggestions.append(unidecode.unidecode(suggestion))
+				continue
+			except ImportError:
+				_log.debug('cannot transliterate external ID suggestion, <unidecode> module not installed')
+			if encoding is None:
+				suggestions.append(suggestion)
+			else:
+				suggestions.append(suggestion.encode(encoding))
+		return suggestions
+
+	#--------------------------------------------------------
 	#--------------------------------------------------------
 	def assimilate_identity(self, other_identity=None, link_obj=None):
 		"""Merge another identity into this one.
@@ -1070,12 +1149,7 @@ where id_identity = %(pat)s and id = %(pk)s"""
 				suffix = u'.gdt'
 			)
 
-		gdt_file = codecs.open (
-			filename = filename,
-			mode = 'wb',
-			encoding = encoding,
-			errors = 'strict'
-		)
+		gdt_file = io.open(filename, mode = 'wt', encoding = encoding, errors = 'strict')
 
 		gdt_file.write(template % (u'013', u'8000', u'6301'))
 		gdt_file.write(template % (u'013', u'9218', u'2.10'))
@@ -1120,9 +1194,10 @@ where id_identity = %(pat)s and id = %(pk)s"""
 		last = etree.SubElement(pat, u'lastname')
 		last.text = gmTools.coalesce(self._payload[self._idx['lastnames']], u'')
 
-		middle = etree.SubElement(pat, u'middlename')
-		middle.set(u'comment', _('preferred name/call name/...'))
-		middle.text = gmTools.coalesce(self._payload[self._idx['preferred']], u'')
+		# privacy
+		#middle = etree.SubElement(pat, u'middlename')
+		#middle.set(u'comment', _('preferred name/call name/...'))
+		#middle.text = gmTools.coalesce(self._payload[self._idx['preferred']], u'')
 
 		pref = etree.SubElement(pat, u'name_prefix')
 		pref.text = gmTools.coalesce(self._payload[self._idx['title']], u'')
@@ -1149,9 +1224,9 @@ where id_identity = %(pat)s and id = %(pk)s"""
 			city.set(u'comment', gmTools.coalesce(adr['suburb'], u''))
 			city.text = gmTools.coalesce(adr['urb'], u'')
 
-			state = etree.SubElement(home, u'state')
-			state.set(u'comment', gmTools.coalesce(adr['l10n_state'], u''))
-			state.text = gmTools.coalesce(adr['code_state'], u'')
+			region = etree.SubElement(home, u'region')
+			region.set(u'comment', gmTools.coalesce(adr['l10n_region'], u''))
+			region.text = gmTools.coalesce(adr['code_region'], u'')
 
 			zipcode = etree.SubElement(home, u'postal_code')
 			zipcode.text = gmTools.coalesce(adr['postcode'], u'')
@@ -1214,8 +1289,9 @@ where id_identity = %(pat)s and id = %(pk)s"""
 		vc.fn.value = self.get_description()
 		vc.add(u'n')
 		vc.n.value = vobject.vcard.Name(family = self._payload[self._idx['lastnames']], given = self._payload[self._idx['firstnames']])
-		vc.add(u'nickname')
-		vc.nickname.value = gmTools.coalesce(self._payload[self._idx['preferred']], u'')
+		# privacy
+		#vc.add(u'nickname')
+		#vc.nickname.value = gmTools.coalesce(self._payload[self._idx['preferred']], u'')
 		vc.add(u'title')
 		vc.title.value = gmTools.coalesce(self._payload[self._idx['title']], u'')
 		vc.add(u'gender')
@@ -1270,7 +1346,7 @@ where id_identity = %(pat)s and id = %(pk)s"""
 			vc_adr = vc.adr.value
 			vc_adr.extended = gmTools.coalesce(home_adr['subunit'], u'')
 			vc_adr.street = gmTools.coalesce(home_adr['street'], u'', u'%s ') + gmTools.coalesce(home_adr['number'], u'')
-			vc_adr.region = gmTools.coalesce(home_adr['l10n_state'], u'')
+			vc_adr.region = gmTools.coalesce(home_adr['l10n_region'], u'')
 			vc_adr.code = gmTools.coalesce(home_adr['postcode'], u'')
 			vc_adr.city = gmTools.coalesce(home_adr['urb'], u'')
 			vc_adr.country = gmTools.coalesce(home_adr['l10n_country'], u'')
@@ -1279,10 +1355,10 @@ where id_identity = %(pat)s and id = %(pk)s"""
 
 		if filename is None:
 			filename = gmTools.get_unique_filename (
-				prefix = u'gm-patient2vcard-',
+				prefix = u'gm-pat2vcf-',
 				suffix = u'.vcf'
 			)
-		vcf = codecs.open(filename, mode = 'wb', encoding = 'utf8')
+		vcf = io.open(filename, mode = 'wt', encoding = 'utf8')
 		vcf.write(vc.serialize().decode('utf-8'))
 		vcf.close()
 
@@ -1484,47 +1560,41 @@ where id_identity = %(pat)s and id = %(pk)s"""
 	#----------------------------------------------------------------------
 	def get_relatives(self):
 		cmd = u"""
-			select
-				t.description,
-				vbp.pk_identity as id,
-				title,
-				firstnames,
-				lastnames,
-				dob,
-				cob,
-				gender,
-				pupic,
-				pk_marital_status,
-				marital_status,
-				xmin_identity,
-				preferred
-			from
-				dem.v_basic_person vbp, dem.relation_types t, dem.lnk_person2relative l
-			where
-				(
-					l.id_identity = %(pk)s and
-					vbp.pk_identity = l.id_relative and
-					t.id = l.id_relation_type
+			SELECT
+				d_rt.description,
+				d_vap.*
+			FROM
+				dem.v_all_persons d_vap,
+				dem.relation_types d_rt,
+				dem.lnk_person2relative d_lp2r
+			WHERE
+				(	d_lp2r.id_identity = %(pk)s
+						AND
+					d_vap.pk_identity = d_lp2r.id_relative
+						AND
+					d_rt.id = d_lp2r.id_relation_type
 				) or (
-					l.id_relative = %(pk)s and
-					vbp.pk_identity = l.id_identity and
-					t.inverse = l.id_relation_type
+					d_lp2r.id_relative = %(pk)s
+						AND
+					d_vap.pk_identity = d_lp2r.id_identity
+						AND
+					d_rt.inverse = d_lp2r.id_relation_type
 				)"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'pk': self.pk_obj}}])
 		if len(rows) == 0:
 			return []
-		return [(row[0], cIdentity(row = {'data': row[1:], 'idx':idx, 'pk_field': 'pk'})) for row in rows]
+		return [(row[0], cPerson(row = {'data': row[1:], 'idx':idx, 'pk_field': 'pk_identity'})) for row in rows]
 	#--------------------------------------------------------
 	def link_new_relative(self, rel_type = 'parent'):
 		# create new relative
 		id_new_relative = create_dummy_identity()
 
-		relative = cIdentity(aPK_obj=id_new_relative)
+		relative = cPerson(aPK_obj=id_new_relative)
 		# pre-fill with data from ourselves
 #		relative.copy_addresses(self)
 		relative.add_name( '**?**', self.get_names()['lastnames'])
 		# and link the two
-		if self._ext_cache.has_key('relatives'):
+		if u'relatives' in self._ext_cache:
 			del self._ext_cache['relatives']
 		cmd = u"""
 			insert into dem.lnk_person2relative (
@@ -1542,7 +1612,7 @@ where id_identity = %(pat)s and id = %(pk)s"""
 	def _get_emergency_contact_from_database(self):
 		if self._payload[self._idx['pk_emergency_contact']] is None:
 			return None
-		return cIdentity(aPK_obj = self._payload[self._idx['pk_emergency_contact']])
+		return cPerson(self._payload[self._idx['pk_emergency_contact']])
 
 	emergency_contact_in_database = property(_get_emergency_contact_from_database, lambda x:x)
 	#----------------------------------------------------------------------
@@ -1661,12 +1731,31 @@ where id_identity = %(pat)s and id = %(pk)s"""
 		return gmProviderInbox.delete_inbox_message(inbox_message = pk)
 	#--------------------------------------------------------
 	def _get_dynamic_hints(self, include_suppressed_needing_invalidation=False):
-		return gmProviderInbox.get_hints_for_patient (
+		return gmAutoHints.get_hints_for_patient (
 			pk_identity = self._payload[self._idx['pk_identity']],
 			include_suppressed_needing_invalidation = include_suppressed_needing_invalidation
 		)
 
 	dynamic_hints = property(_get_dynamic_hints, lambda x:x)
+	#--------------------------------------------------------
+	def _get_suppressed_hints(self):
+		return gmAutoHints.get_suppressed_hints(pk_identity = self._payload[self._idx['pk_identity']])
+
+	suppressed_hints = property(_get_suppressed_hints, lambda x:x)
+
+	#--------------------------------------------------------
+	def _get_primary_provider_identity(self):
+		if self._payload[self._idx['pk_primary_provider']] is None:
+			return None
+		cmd = u"SELECT * FROM dem.v_all_persons WHERE pk_identity = (SELECT pk_identity FROM dem.v_staff WHERE pk_staff = %(pk_staff)s)"
+		args = {'pk_staff': self._payload[self._idx['pk_primary_provider']]}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		if len(rows) == 0:
+			return None
+		return cPerson(row = {'data': rows[0], 'idx': idx, 'pk_field': 'pk_identity'})
+
+	primary_provider_identity = property(_get_primary_provider_identity, lambda x:x)
+
 	#--------------------------------------------------------
 	def _get_primary_provider(self):
 		if self._payload[self._idx['pk_primary_provider']] is None:
@@ -1675,15 +1764,18 @@ where id_identity = %(pat)s and id = %(pk)s"""
 		return gmStaff.cStaff(aPK_obj = self._payload[self._idx['pk_primary_provider']])
 
 	primary_provider = property(_get_primary_provider, lambda x:x)
+
 	#----------------------------------------------------------------------
 	# convenience
 	#----------------------------------------------------------------------
 	def get_dirname(self):
 		"""Format patient demographics into patient specific path name fragment."""
-		return (u'%s-%s%s-%s' % (
+		#return (u'%s-%s%s-%s' % (
+		return (u'%s-%s-%s' % (
 			self._payload[self._idx['lastnames']].replace(u' ', u'_'),
 			self._payload[self._idx['firstnames']].replace(u' ', u'_'),
-			gmTools.coalesce(self._payload[self._idx['preferred']], u'', template_initial = u'-(%s)').replace(u' ', u'_'),
+			# privacy
+			#gmTools.coalesce(self._payload[self._idx['preferred']], u'', template_initial = u'-(%s)').replace(u' ', u'_'),
 			self.get_formatted_dob(format = '%Y-%m-%d', encoding = gmI18N.get_encoding())
 		)).replace (
 			u"'", u""
@@ -1762,14 +1854,14 @@ def set_yielder(yielder):
 	_log.debug('setting yielder to <%s>', yielder)
 
 #============================================================
-class cPatient(cIdentity):
+class cPatient(cPerson):
 	"""Represents a person which is a patient.
 
-	- a specializing subclass of cIdentity turning it into a patient
+	- a specializing subclass of cPerson turning it into a patient
 	- its use is to cache subobjects like EMR and document folder
 	"""
 	def __init__(self, aPK_obj=None, row=None):
-		cIdentity.__init__(self, aPK_obj = aPK_obj, row = row)
+		cPerson.__init__(self, aPK_obj = aPK_obj, row = row)
 		self.__emr_access_lock = threading.Lock()
 		self.__emr = None
 		self.__doc_folder = None
@@ -1783,7 +1875,7 @@ class cPatient(cIdentity):
 			self.__emr.cleanup()
 		if self.__doc_folder is not None:
 			self.__doc_folder.cleanup()
-		cIdentity.cleanup(self)
+		cPerson.cleanup(self)
 	#----------------------------------------------------------------
 	def ensure_has_allergy_state(self, pk_encounter=None):
 		from Gnumed.business.gmAllergy import ensure_has_allergy_state
@@ -1853,10 +1945,12 @@ class cPatient(cIdentity):
 
 #============================================================
 class gmCurrentPatient(gmBorg.cBorg):
-	"""Patient Borg to hold currently active patient.
+	"""Patient Borg to hold the currently active patient.
 
 	There may be many instances of this but they all share state.
 
+	The underlying dem.identity row must have .deleted set to FALSE.
+
 	The sequence of events when changing the active patient:
 
 		1) Registered callbacks are run.
@@ -1870,6 +1964,9 @@ class gmCurrentPatient(gmBorg.cBorg):
 
 		2) Signal "pre_patient_unselection" is sent.
 			This does not wait for nor check results.
+			The keyword pk_identity contains the
+			PK of the person being switched away
+			from.
 
 		3) the current patient is unset (gmNull.cNull)
 
@@ -1944,6 +2041,10 @@ class gmCurrentPatient(gmBorg.cBorg):
 		if (self.patient['pk_identity'] == patient['pk_identity']) and not forced_reload:
 			return None
 
+		if patient['is_deleted']:
+			_log.error('cannot set active patient to disabled dem.identity row: %s', patient)
+			raise ValueError('gmPerson.gmCurrentPatient.__init__(): <patient> is disabled: %s' % patient)
+
 		# user wants different patient
 		_log.info('patient change [%s] -> [%s] requested', self.patient['pk_identity'], patient['pk_identity'])
 
@@ -2149,6 +2250,7 @@ class cMatchProvider_Provider(gmMatchProvider.cMatchProvider_SQL2):
 			]
 		)
 		self.setThresholds(1, 2, 3)
+
 #============================================================
 # convenience functions
 #============================================================
@@ -2180,10 +2282,20 @@ INSERT INTO dem.names (
 		],
 		return_data = True
 	)
-	ident = cIdentity(aPK_obj = rows[0][0])
+	ident = cPerson(aPK_obj = rows[0][0])
 	gmHooks.run_hook_script(hook = u'post_person_creation')
 	return ident
 
+#============================================================
+def disable_identity(pk_identity):
+	_log.info('disabling identity [%s]', pk_identity)
+	cmd = u"UPDATE dem.identity SET deleted = true WHERE pk = %(pk)s"
+	args = {'pk': identity['pk_identity']}
+	gmPG2.run_rw_queries(queries = [
+		{'cmd': cmd, 'args': args}
+	])
+	return True
+
 #============================================================
 def create_dummy_identity():
 	cmd = u"INSERT INTO dem.identity(gender) VALUES (NULL::text) RETURNING pk"
@@ -2191,7 +2303,7 @@ def create_dummy_identity():
 		queries = [{'cmd': cmd}],
 		return_data = True
 	)
-	return gmDemographicRecord.cIdentity(aPK_obj = rows[0][0])
+	return gmDemographicRecord.cPerson(aPK_obj = rows[0][0])
 
 #============================================================
 def identity_exists(pk_identity):
@@ -2211,7 +2323,7 @@ def set_active_patient(patient=None, forced_reload=False):
 
 	if isinstance(patient, cPatient):
 		pat = patient
-	elif isinstance(patient, cIdentity):
+	elif isinstance(patient, cPerson):
 		pat = pat.as_patient
 	elif patient == -1:
 		pat = patient
@@ -2219,7 +2331,7 @@ def set_active_patient(patient=None, forced_reload=False):
 		# maybe integer ?
 		success, pk = gmTools.input2int(initial = patient, minval = 1)
 		if not success:
-			raise ValueError('<patient> must be either -1, >0, or a cPatient, cIdentity or gmCurrentPatient instance, is: %s' % patient)
+			raise ValueError('<patient> must be either -1, >0, or a cPatient, cPerson or gmCurrentPatient instance, is: %s' % patient)
 		# but also valid patient ID ?
 		try:
 			pat = cPatient(aPK_obj = pk)
@@ -2235,6 +2347,7 @@ def set_active_patient(patient=None, forced_reload=False):
 		return False
 
 	return True
+
 #============================================================
 # gender related
 #------------------------------------------------------------
@@ -2352,7 +2465,7 @@ def get_person_IDs():
 
 #============================================================
 def get_persons_from_pks(pks=None):
-	return [ cIdentity(aPK_obj = pk) for pk in pks ]
+	return [ cPerson(aPK_obj = pk) for pk in pks ]
 #============================================================
 def get_person_from_xdt(filename=None, encoding=None, dob_format=None):
 	from Gnumed.business import gmXdtObjects
@@ -2382,7 +2495,7 @@ if __name__ == '__main__':
 	#--------------------------------------------------------
 	def test_set_active_pat():
 
-		ident = cIdentity(1)
+		ident = cPerson(1)
 		print "setting active patient with", ident
 		set_active_patient(patient=ident)
 
@@ -2427,7 +2540,7 @@ if __name__ == '__main__':
 		new_identity['title'] = 'test title';
 		new_identity['gender'] = 'f';
 		new_identity.save_payload()
-		print 'Refetching identity from db: %s' % cIdentity(aPK_obj=new_identity['pk_identity'])
+		print 'Refetching identity from db: %s' % cPerson(aPK_obj=new_identity['pk_identity'])
 
 		print '\nGetting all names...'
 		for a_name in new_identity.get_names():
@@ -2476,18 +2589,18 @@ if __name__ == '__main__':
 			print "%s, %s, %s" % (gender[idx['tag']], gender[idx['l10n_label']], gender[idx['sort_weight']])
 	#--------------------------------------------------------
 	def test_export_area():
-		person = cIdentity(aPK_obj = 12)
+		person = cPerson(aPK_obj = 12)
 		print person
 		print person.export_area
 		print person.export_area.items
 	#--------------------------------------------------------
 	def test_ext_id():
-		person = cIdentity(aPK_obj = 9)
+		person = cPerson(aPK_obj = 9)
 		print person.get_external_ids(id_type=u'Fachgebiet', issuer=u'Ärztekammer')
 		#print person.get_external_ids()
 	#--------------------------------------------------------
 	def test_vcf():
-		person = cIdentity(aPK_obj = 12)
+		person = cPerson(aPK_obj = 12)
 		print person.export_as_vcard()
 
 	#--------------------------------------------------------
@@ -2496,6 +2609,10 @@ if __name__ == '__main__':
 		print "pat.get_emr()", pat.get_emr()
 
 	#--------------------------------------------------------
+	def test_ext_id():
+		person = cPerson(aPK_obj = 12)
+		print person.suggest_external_id(target = u'Orthanc')
+	#--------------------------------------------------------
 	#test_dto_person()
 	#test_identity()
 	#test_set_active_pat()
@@ -2511,6 +2628,7 @@ if __name__ == '__main__':
 	#test_export_area()
 	#test_ext_id()
 	#test_vcf()
+	test_ext_id()
 	test_current_patient()
 
 #============================================================
